@@ -1,6 +1,6 @@
 /* channel_commands.c
  *
- * $Id: channel_commands.c,v 1.22 2000/11/26 17:40:00 nisse Exp $
+ * $Id: channel_commands.c,v 1.24 2001/07/04 19:46:10 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -52,7 +52,7 @@ do_channel_open_command(struct command *s,
   if (index < 0)
     {
       /* We have run out of channel numbers. */
-      werror("do_channel_open_command: alloc_channel() failed\n");
+      werror("do_channel_open_command: alloc_channel failed\n");
       EXCEPTION_RAISE(e,
 		      make_channel_open_exception(
 			SSH_OPEN_RESOURCE_SHORTAGE,
@@ -116,16 +116,17 @@ do_channel_global_command(struct command *s,
   C_WRITE(connection, request);
 }
 
-struct lsh_object *
-do_install_global_request_handler(struct collect_info_2 *info,
-				  struct lsh_object *a,
-				  struct lsh_object *b)
+void
+do_install_global_request_handler(struct command_2 *s,
+				  struct lsh_object *a1,
+				  struct lsh_object *a2,
+				  struct command_continuation *c,
+				  struct exception_handler *e UNUSED)
 {
-  CAST(install_info, self, info);
-  CAST(ssh_connection, connection, a);
-  CAST_SUBTYPE(global_request, handler, b);
+  CAST(install_info, self, s);
+  CAST(ssh_connection, connection, a1);
+  CAST_SUBTYPE(global_request, handler, a2);
 
-  assert(!info->next);
   assert(handler);
 
   trace("Installing global request handler for '%a'\n", self->name);
@@ -134,19 +135,20 @@ do_install_global_request_handler(struct collect_info_2 *info,
 	    self->name,
 	    &handler->super);
 
-  return a;
+  COMMAND_RETURN(c, connection);
 }
 
-struct lsh_object *
-do_install_channel_open_handler(struct collect_info_2 *info,
-				struct lsh_object *a,
-				struct lsh_object *b)
+void
+do_install_channel_open_handler(struct command_2 *s,
+				struct lsh_object *a1,
+				struct lsh_object *a2,
+				struct command_continuation *c,
+				struct exception_handler *e UNUSED)
 {
-  CAST(install_info, self, info);
-  CAST(ssh_connection, connection, a);
-  CAST_SUBTYPE(channel_open, handler, b);
+  CAST(install_info, self, s);
+  CAST(ssh_connection, connection, a1);
+  CAST_SUBTYPE(channel_open, handler, a2);
 
-  assert(!info->next);
   assert(handler);
   
   trace("Installing channel open handler for '%a'\n", self->name);
@@ -155,9 +157,10 @@ do_install_channel_open_handler(struct collect_info_2 *info,
 	    self->name,
 	    &handler->super);
 
-  return a;
+  COMMAND_RETURN(c, connection);
 }
 
+
 /* Special cases, when the handler is known early */
  
 /* Takes a connection as argument, and installs a fix handler */
