@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.120 2001/06/26 16:31:42 nisse Exp $ */
+ * $Id: client.c,v 1.122 2001/08/12 20:21:43 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -423,23 +423,31 @@ init_client_options(struct client_options *self,
 }
 
 /* Host to connect to */
-DEFINE_COMMAND_SIMPLE(client_options2remote, a)
+DEFINE_COMMAND(client_options2remote)
+     (struct command *s UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
   CAST_SUBTYPE(client_options, options, a);
   trace("client.c: client_options2remote\n");
   
-  return &options->remote->super;
+  COMMAND_RETURN(c, options->remote);
 }
 
 /* Host to connect to */
-DEFINE_COMMAND_SIMPLE(client_options2actions, a)
+DEFINE_COMMAND(client_options2actions)
+     (struct command *s UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
   CAST_SUBTYPE(client_options, options, a);
 
   trace("client.c: client_options2actions, %i actions\n",
 	options->actions.length);
   
-  return &queue_to_list(&options->actions)->super.super;
+  COMMAND_RETURN(c, queue_to_list(&options->actions));
 }
 
 static const struct argp_option
@@ -456,6 +464,7 @@ client_options[] =
 #endif
   { "nop", 'N', NULL, 0, "No operation (suppresses the default action, "
     "which is to spawn a remote shell)", 0 },
+  { "background", 'B', NULL, 0, "Put process into the background. Implies -N.", 0 },
   { "execute", 'E', "command", 0, "Execute a command on the remote machine", 0 },
   { "shell", 'S', "command", 0, "Spawn a remote shell", 0 },
   /* { "gateway", 'G', NULL, 0, "Setup a local gateway", 0 }, */
@@ -698,6 +707,42 @@ fork_output(int out)
     }
 }
 
+/* A callback that exits the process immediately. */
+static void
+do_exit(struct lsh_callback *self UNUSED)
+{
+  exit(EXIT_SUCCESS);
+}
+
+/* FIXME: Use const? */
+static struct lsh_callback
+exit_callback = { STATIC_HEADER, do_exit };
+
+DEFINE_COMMAND(background_process)
+     (struct command *s UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
+{
+  switch (fork())
+    {
+    case 0:
+      /* Child */
+      /* FIXME: Should we create a new process group, close our tty
+       * and stdio, etc? */
+      COMMAND_RETURN(c, a);
+      break;
+    case -1:
+      /* Error */
+      werror("background_process: fork failed (errno = %i): %z\n",
+	     errno, STRERROR(errno));
+      COMMAND_RETURN(c, a);
+      break;
+    default:
+      /* Parent */
+      _exit(EXIT_SUCCESS);
+    }
+}
 
 /* Create a session object. stdout and stderr are shared (although
  * with independent lsh_fd objects). stdin can be used by only one
@@ -753,7 +798,11 @@ make_client_session(struct client_options *options)
 
   /* Bind ^Z to suspend. */
   if (escape)
-    escape->dispatch[26] = &suspend_callback;
+    {
+      /* Bind ^Z to suspend. */
+      escape->dispatch[26] = &suspend_callback;
+      escape->dispatch['.'] = &exit_callback;
+    }
   
   debug("lsh.c: Setting up stdout\n");
 
@@ -1007,6 +1056,11 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
       options->start_shell = 0;
       break;
 
+    case 'B':
+      options->start_shell = 0;
+      client_add_action(options, &background_process);
+      break;
+      
     CASE_FLAG('g', with_remote_peers);
 
 #if WITH_PTY_SUPPORT
