@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: client.h,v 1.40 2001/06/19 21:15:40 nisse Exp $ */
+ * $Id: client.h,v 1.41 2001/07/06 18:07:31 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -165,11 +165,11 @@ client_parse_forward_arg(char *arg,
 			 UINT32 *listen_port,
 			 struct address_info **target);
 
-extern struct command_simple client_options2remote;
-#define OPTIONS2REMOTE (&client_options2remote.super.super)
+extern struct command client_options2remote;
+#define OPTIONS2REMOTE (&client_options2remote.super)
 
-extern struct command_simple client_options2actions;
-#define OPTIONS2ACTIONS (&client_options2actions.super.super)
+extern struct command client_options2actions;
+#define OPTIONS2ACTIONS (&client_options2actions.super)
 
 extern const struct argp client_argp;
 
