@@ -1,6 +1,6 @@
 /* client_keyexchange.c
  *
- * $Id: client_keyexchange.c,v 1.56 2001/02/04 15:04:10 nisse Exp $ */
+ * $Id: client_keyexchange.c,v 1.58 2001/04/04 21:50:47 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -71,10 +71,10 @@ do_handle_dh_reply(struct packet_handler *c,
   struct verifier *v;
   int res;
   
-  trace("handle_dh_reply()\n");
+  trace("handle_dh_reply\n");
 
   server_key = dh_process_server_msg(&closure->dh, &signature, packet);
-  lsh_string_free(packet);
+
   if (!server_key)
     {
       disconnect_kex_failed(connection, "Bad dh-reply\r\n");
@@ -204,7 +204,6 @@ do_srp_client_proof_handler(struct packet_handler *s,
 
   int res = srp_process_server_proof(self->srp->m2,
 				     packet);
-  lsh_string_free(packet);
 
   connection->dispatch[SSH_MSG_KEXSRP_PROOF] = &connection_fail_handler;
   
