@@ -1,6 +1,6 @@
 /* connection_commands.h
  *
- * $Id: connection_commands.h,v 1.16 2000/11/15 16:39:01 nisse Exp $
+ * $Id: connection_commands.h,v 1.18 2001/07/06 18:10:01 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -38,15 +38,15 @@
        (connection object ssh_connection)))
 */
 
-extern struct command_simple connection_remember;
+extern struct command_2 connection_remember;
 #define CONNECTION_REMEMBER (&connection_remember.super.super)
 
 struct command *
 make_connection_if_srp(struct command *then_f,
 		       struct command *else_f);
 
-extern struct collect_info_1 connection_if_srp_command;
-#define CONNECTION_IF_SRP (&connection_if_srp_command.super.super.super)
+extern struct command_3 connection_if_srp;
+#define CONNECTION_IF_SRP (&connection_if_srp_command.super.super)
 
 extern struct command connection_require_userauth;
 #define CONNECTION_REQUIRE_USERAUTH (&connection_require_userauth.super)
