@@ -2,7 +2,7 @@
  *
  * Interact with the user.
  *
- * $Id: interact.h,v 1.4 2000/11/15 21:58:26 nisse Exp $*/
+ * $Id: interact.h,v 1.8 2001/04/26 04:46:04 nisse Exp $*/
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -39,14 +39,6 @@ struct terminal_dimensions;
 /* Abstract class defining methods needed to communicate with the
  * user's terminal. */
 
-struct terminal_dimensions
-{
-  UINT32 char_width;
-  UINT32 char_height;
-  UINT32 pixel_width;
-  UINT32 pixel_height;
-};
-
 /* GABA:
    (class
      (name terminal_attributes)
@@ -73,12 +65,13 @@ struct terminal_dimensions
      (vars
        (is_tty method int)
        ; (read_line method int "UINT32 size" "UINT8 *buffer")
+       ;; FIXME: Do we really need the FREE parameter?
        (read_password method (string)
                   "UINT32 max_length"
-                  "struct lsh_string *prompt"
+                  "const struct lsh_string *prompt"
 		  "int free")
        (yes_or_no method int
-                  "struct lsh_string *prompt"
+                  "const struct lsh_string *prompt"
 		  "int def" "int free")
 
        (get_attributes method (object terminal_attributes) )
@@ -104,14 +97,6 @@ struct terminal_dimensions
 #define INTERACT_WINDOW_SUBSCRIBE(i, c) \
   ((i)->window_change_subscribe((i), (c)))
 
-#if 0
-extern int tty_fd;
-
-int lsh_open_tty(void);
-int tty_read_line(UINT32 size, UINT8 *buffer);
-int yes_or_no(struct lsh_string *s, int def, int free);
-#endif
-
 struct interact *
 make_unix_interact(struct io_backend *backend);
 
