@@ -2,7 +2,7 @@
  *
  * Decode ssh2 keys.
  *
- * $Id: lsh-decode-key.c,v 1.6 2000/11/15 17:01:41 nisse Exp $
+ * $Id: lsh-decode-key.c,v 1.9 2001/04/08 19:00:10 nisse Exp $
  */
 
 #include "dsa.h"
@@ -10,6 +10,7 @@
 #include "lsh_argp.h"
 #include "publickey_crypto.h"
 #include "read_file.h"
+#include "rsa.h"
 #include "sexp_commands.h"
 #include "spki.h"
 #include "version.h"
@@ -156,7 +157,7 @@ do_decode_key(struct abstract_write *s,
 	    {
 	      struct verifier *v;
 	    
-	      werror("Reading key of type ssh-dss...\n");
+	      werror("lsh-decode-key: Reading key of type ssh-dss...\n");
 
 	      if (! ( (v = parse_ssh_dss_public(&buffer)) ))
 		{
@@ -169,6 +170,23 @@ do_decode_key(struct abstract_write *s,
 	      COMMAND_RETURN(self->c, spki_make_public_key(v));
 	      break;
 	    }
+	  case ATOM_SSH_RSA:
+	    {
+	      struct verifier *v;
+	    
+	      werror("lsh-decode-key: Reading key of type ssh-rsa...\n");
+
+	      if (! ( (v = parse_ssh_rsa_public(&buffer)) ))
+		{
+		  EXCEPTION_RAISE(self->e,
+				  make_simple_exception(EXC_APP_BAD_KEY,
+							"Invalid rsa key.\n"));
+		  return;
+		}
+
+	      COMMAND_RETURN(self->c, spki_make_public_key(v));
+	      break;
+	    }	    
 	  default:
 	    EXCEPTION_RAISE(self->e,
 			    make_simple_exception(EXC_APP_BAD_KEY,
@@ -216,7 +234,8 @@ int main(int argc, char **argv)
 
   argp_parse(&main_argp, argc, argv, 0, NULL, options);
 
-  in = make_lsh_fd(backend, STDIN_FILENO, &exc_handler);
+  in = make_lsh_fd(backend, STDIN_FILENO, "stdin",
+		   &exc_handler);
 
   /* We want an exception handler that deals with EXC_FINISH_IO. */
   e = make_exc_finish_read_handler(in, &exc_handler, HANDLER_CONTEXT);
@@ -226,7 +245,8 @@ int main(int argc, char **argv)
 			O_WRONLY | O_CREAT, 0666,
 			BLOCKSIZE, NULL, e);
   else
-    out = io_write(make_lsh_fd(backend, STDOUT_FILENO, e),
+    out = io_write(make_lsh_fd(backend, STDOUT_FILENO,
+			       "stdout", e),
 		   BLOCKSIZE, NULL);
   io_read
     (in,
