@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.140 2001/04/17 19:05:51 nisse Exp $ */
+ * $Id: lsh.c,v 1.143 2001/08/21 11:14:44 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -68,17 +68,17 @@
 #include "lsh_argp.h"
 
 /* Forward declarations */
-struct command_simple lsh_verifier_command;
+struct command_2 lsh_verifier_command;
 #define OPTIONS2VERIFIER (&lsh_verifier_command.super.super)
 
-struct command_simple lsh_login_command;
+struct command_2 lsh_login_command;
 #define LSH_LOGIN (&lsh_login_command.super.super)
 
 static struct command options2known_hosts;
 #define OPTIONS2KNOWN_HOSTS (&options2known_hosts.super)
 
-struct command_simple options2service;
-#define OPTIONS2SERVICE (&options2service.super.super)
+struct command options2service;
+#define OPTIONS2SERVICE (&options2service.super)
 
 static struct command options2identities;
 #define OPTIONS2IDENTITIES (&options2identities.super)
@@ -87,9 +87,6 @@ static struct request_service request_userauth_service =
 STATIC_REQUEST_SERVICE(ATOM_SSH_USERAUTH);
 #define REQUEST_USERAUTH_SERVICE (&request_userauth_service.super.super)
 
-static struct request_service request_connection_service =
-STATIC_REQUEST_SERVICE(ATOM_SSH_CONNECTION);
-#define REQUEST_CONNECTION_SERVICE (&request_connection_service.super.super)
 
 #include "lsh.c.x"
 
@@ -119,9 +116,6 @@ STATIC_REQUEST_SERVICE(ATOM_SSH_CONNECTION);
        (with_srp_keyexchange . int)
        (with_dh_keyexchange . int)
 
-       ; Ask for the userauth service
-       (with_userauth . int)
-
        ; Command to invoke to start ssh-connection service)
        (service object command)
        
@@ -169,8 +163,6 @@ make_options(struct io_backend *backend,
 
   /* By default, enable only one of dh and srp. */
   self->with_dh_keyexchange = -1;
-
-  self->with_userauth = -1;
   
   self->kex_algorithms = NULL;
   
@@ -180,10 +172,14 @@ make_options(struct io_backend *backend,
 
 /* Request ssh-userauth or ssh-connection service, as appropriate,
  * and pass the options as a first argument. */
-DEFINE_COMMAND_SIMPLE(options2service, a)
+DEFINE_COMMAND(options2service)
+     (struct command *s UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
   CAST(lsh_options, options, a);
-  return &options->service->super;
+  COMMAND_RETURN(c, options->service);
 }
 
 /* Open hostkey database. By default, "~/.lsh/known_hosts". */
@@ -278,27 +274,6 @@ do_options2identities(struct command *ignored UNUSED,
 static struct command options2identities =
 STATIC_COMMAND(do_options2identities);
 
-/* GABA:
-   (class
-     (name options_command)
-     (super command)
-     (vars
-       (options object lsh_options)))
-*/
-
-static struct command *
-make_options_command(struct lsh_options *options,
-		     void (*call)(struct command *s,
-				  struct lsh_object *a,
-				  struct command_continuation *c,
-				  struct exception_handler *e))
-{
-  NEW(options_command, self);
-  self->super.call = call;
-  self->options = options;
-
-  return &self->super;
-}
 
 /* Maps a host key to a (trusted) verifier object. */
 
@@ -492,70 +467,60 @@ make_lsh_host_db(struct spki_context *db,
   return &res->super;
 }
 
-/* Takes an spki_context as argument and returns a lookup_verifier */
-static void
-do_lsh_verifier(struct command *s,
-		struct lsh_object *a,
-		struct command_continuation *c,
-		struct exception_handler *e UNUSED)
+/* Takes options and an spki_context as argument and returns a
+ * lookup_verifier */
+
+DEFINE_COMMAND2(lsh_verifier_command)
+     (struct command_2 *s UNUSED,
+      struct lsh_object *a1,
+      struct lsh_object *a2,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
-  CAST(options_command, self, s);
-  CAST_SUBTYPE(spki_context, db, a);
+  CAST(lsh_options, options, a1);
+  CAST_SUBTYPE(spki_context, db, a2);
   COMMAND_RETURN(c, make_lsh_host_db(db,
-				     self->options->super.tty,
-				     self->options->super.remote,
-				     self->options->sloppy,
-				     self->options->capture_file));
+				     options->super.tty,
+				     options->super.remote,
+				     options->sloppy,
+				     options->capture_file));
 }
 
-/* Takes an options object as argument and returns a lookup_verifier */
 
-DEFINE_COMMAND_SIMPLE(lsh_verifier_command, a)
+/* (login options public-keys connection) */
+DEFINE_COMMAND2(lsh_login_command)
+     (struct command_2 *s UNUSED,
+      struct lsh_object *a1,
+      struct lsh_object *a2,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
-  CAST(lsh_options, options, a);
-
-  return
-    & make_options_command(options,
-			   do_lsh_verifier)->super;
-}
-
-/* list-of-public-keys -> login-command */
-static void
-do_lsh_login(struct command *s,
-	     struct lsh_object *a,
-	     struct command_continuation *c,
-	     struct exception_handler *e UNUSED)
-{
-  CAST(options_command, self, s);
-  CAST_SUBTYPE(object_list, keys, a);
+  CAST(lsh_options, options, a1);
+  CAST_SUBTYPE(object_list, keys, a2);
 
   struct client_userauth_method *password
-    = make_client_password_auth(self->options->super.tty);
+    = make_client_password_auth(options->super.tty);
+
+  /* FIXME: Perhaps we should try "none" only when using SRP. */
+  struct client_userauth_method *none
+    = make_client_none_auth();
   
   COMMAND_RETURN(c,
 		 make_client_userauth
-		 (ssh_format("%lz", self->options->super.user),
+		 (ssh_format("%lz", options->super.user),
 		  ATOM_SSH_CONNECTION,
 		  (LIST_LENGTH(keys)
 		   ? make_object_list
-		   (2, 
+		   (3,
+                    none,
 		    make_client_publickey_auth(keys),
 		    password,
 		    -1)
-		   : make_object_list(1, password, -1))));
+		   : make_object_list(2, none, password, -1))));
 }
 
-/* (login options public-keys connection) */
-DEFINE_COMMAND_SIMPLE(lsh_login_command, a)
-{
-  CAST(lsh_options, options, a);
 
-  return
-    &make_options_command(options,
-			  do_lsh_login)->super;
-}
-
-/* NOTE: options2identities is a command_simple, so it must not be
+/* NOTE: options2identities can block for reading, so it must not be
  * invoked directly. */
 
 /* Requests the ssh-userauth service, log in, and request connection
@@ -569,8 +534,8 @@ DEFINE_COMMAND_SIMPLE(lsh_login_command, a)
        (lambda (connection)
          (lsh_login options
 	   
-	   ; The prog1 delay is needed because options2identities is
-	   ; not a command_simple.
+	   ; The prog1 delay is needed because options2identities
+	   ; may not return immediately.
 	   (options2identities (prog1 options connection))
 
 	   ; Request the userauth service
@@ -620,7 +585,6 @@ const char *argp_program_bug_address = BUG_ADDRESS;
 
 #define OPT_DH 0x206
 #define OPT_SRP 0x207
-#define OPT_USERAUTH 0x208
 
 #define OPT_STDIN 0x210
 #define OPT_STDOUT 0x211
@@ -655,11 +619,13 @@ main_options[] =
 
   { "no-dh-keyexchange", OPT_DH | ARG_NOT, NULL, 0, "Disable DH support.", 0 },
 
+#if 0
   { "userauth", OPT_USERAUTH, NULL, 0,
     "Request the ssh-userauth service (default, unless SRP is being used).", 0 },
   { "no-userauth", OPT_USERAUTH | ARG_NOT, NULL, 0,
     "Request the ssh-userauth service (default if SRP is used).", 0 },
-
+#endif
+  
   /* ACtions */
   { "forward-remote-port", 'R', "remote-port:target-host:target-port",
     0, "", CLIENT_ARGP_ACTION_GROUP },
@@ -761,49 +727,8 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	argp_error(state, "All keyexchange algorithms disabled.");
 
       {
-	struct command *perform_userauth = NULL;
-
-	/* NOTE: The default, self->with_userauth < 0, means that we
-	 * should figure out the right thing automatically. */
-
-	if (self->with_userauth < 0)
-	  {
-	    /* Make the decision  early, if possible. */
-	    if (!self->with_srp_keyexchange)
-	      /* We're not using SRP, so we request the
-	       * ssh-userauth-service */
-	      self->with_userauth = 1;
-
-	    else if (!self->with_dh_keyexchange)
-	      /* We're using SRP, and should not need any extra
-	       * user authentication. */
-	      self->with_userauth = 0;
-	  }
-	   
-	if (self->with_userauth)
-	  {
-	    CAST_SUBTYPE(command, o, make_lsh_userauth(self));
-	    perform_userauth = o;
-	  }
-
-	switch(self->with_userauth)
-	  {
-	  case 0:
-	    self->service = &request_connection_service.super;
-	    break;
-	  case 1:
-	    self->service = perform_userauth;
-	    break;
-	  case -1:
-	    /* Examine the CONNECTION_SRP flag, later. */
-	    self->service
-	      = make_connection_if_srp(&request_connection_service.super,
-				       perform_userauth);
-	  default:
-	    fatal("Internal error.\n");
-	  }
-	  
-	assert(self->service);
+        CAST_SUBTYPE(command, o, make_lsh_userauth(self));
+        self->service = o;
       }
 	
       {
@@ -903,8 +828,6 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
     CASE_FLAG(OPT_DH, with_dh_keyexchange);
     CASE_FLAG(OPT_SRP, with_srp_keyexchange);
 
-    CASE_FLAG(OPT_USERAUTH, with_userauth);
-      
     case 'R':
       {
 	UINT32 listen_port;
