@@ -1,6 +1,6 @@
 /* lshg.c
  *
- * $Id: lshg.c,v 1.10 2001/04/17 19:39:38 nisse Exp $
+ * $Id: lshg.c,v 1.12 2001/08/12 20:27:00 nisse Exp $
  *
  * Connect to a gateway. */
 
@@ -43,8 +43,8 @@
 #include "lsh_argp.h"
 
 /* Forward declarations */
-struct command_simple options2info;
-#define OPTIONS2INFO (&options2info.super.super)
+struct command options2info;
+#define OPTIONS2INFO (&options2info.super)
 
 
 #include "lshg.c.x"
@@ -70,12 +70,15 @@ make_options(struct io_backend *backend,
   return self;
 }
 
-DEFINE_COMMAND_SIMPLE(options2info, a)
+DEFINE_COMMAND(options2info)
+     (struct command *s UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
   CAST(lshg_options, self, a);
-  return &make_gateway_address(self->super.local_user,
-			       self->super.user,
-			       self->super.remote)->super;
+
+  COMMAND_RETURN(c, self->gateway);
 }
 
 
@@ -206,6 +209,12 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 					   self->super.user,
 					   self->super.remote);
 
+      if (!self->gateway)
+	{
+	  argp_error(state, "Local or remote user name, or the target host name, are too "
+		     "strange for the gateway socket name construction.");
+	  break;
+	}
       break;
 
     case 'D':
