@@ -1,6 +1,6 @@
 /* server.c
  *
- * $Id: server.c,v 1.64 2000/10/31 22:33:27 nisse Exp $ */
+ * $Id: server.c,v 1.66 2001/04/04 21:50:48 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -34,7 +34,6 @@
 #include "read_line.h"
 #include "read_packet.h"
 #include "reaper.h"
-#include "service.h"
 #include "ssh.h"
 #include "unpad.h"
 #include "werror.h"
@@ -47,6 +46,22 @@
 
 #include "server.c.x"
 
+
+static struct lsh_string *
+format_service_accept(int name)
+{
+  return ssh_format("%c%a", SSH_MSG_SERVICE_ACCEPT, name);
+}
+
+#if DATAFELLOWS_WORKAROUNDS	      
+static struct lsh_string *
+format_service_accept_kludge(void)
+{
+  return ssh_format("%c", SSH_MSG_SERVICE_ACCEPT);
+}
+#endif /* DATAFELLOWS_WORKAROUNDS */
+
+
 /* GABA:
    (class
      (name service_handler)
@@ -57,13 +72,6 @@
        (e object exception_handler)))
 */
 
-#if DATAFELLOWS_WORKAROUNDS	      
-static struct lsh_string *format_service_accept_kludge(void)
-{
-  return ssh_format("%c", SSH_MSG_SERVICE_ACCEPT);
-}
-#endif /* DATAFELLOWS_WORKAROUNDS */
-
 static void
 do_service_request(struct packet_handler *c,
 		   struct ssh_connection *connection,
@@ -82,8 +90,6 @@ do_service_request(struct packet_handler *c,
       && parse_atom(&buffer, &name)
       && parse_eod(&buffer))
     {
-      lsh_string_free(packet);
-      
       if (name)
 	{
 	  CAST_SUBTYPE(command, service, ALIST_GET(closure->services, name));
@@ -111,10 +117,7 @@ do_service_request(struct packet_handler *c,
       
     }
   else
-    {
-      lsh_string_free(packet);
-      PROTOCOL_ERROR(connection->e, "Invalid SERVICE_REQUEST message");
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid SERVICE_REQUEST message");
 }
 
 static struct packet_handler *
