@@ -2,7 +2,7 @@
  *
  * user authorization database
  *
- * $Id: server_authorization.c,v 1.15 2001/01/31 18:04:36 nisse Exp $ */
+ * $Id: server_authorization.c,v 1.16 2001/04/08 15:15:14 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -87,12 +87,12 @@ do_key_lookup(struct lookup_verifier *c,
   
   /* FIXME: Proper spki acl reading should go here. */
   
-  filename = ssh_cformat(".lsh/%lS/%lxfS", 
-			 closure->index_name,
-			 hash_string(closure->hashalgo,
-				     sexp_format(spki_make_public_key(v),
-						 SEXP_CANONICAL, 0),
-				     1));
+  filename = ssh_format(".lsh/%lS/%lxfS", 
+			closure->index_name,
+			hash_string(closure->hashalgo,
+				    sexp_format(spki_make_public_key(v),
+						SEXP_CANONICAL, 0),
+				    1));
   
   if (USER_FILE_EXISTS(keyholder, filename, 1))
     return v;
