@@ -1,6 +1,6 @@
 /* server_session.h
  *
- * $Id: server_session.h,v 1.12 2000/06/01 10:42:00 nisse Exp $
+ * $Id: server_session.h,v 1.15 2001/08/26 14:39:35 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -35,19 +35,19 @@
 struct ssh_channel *
 make_server_session(UINT32 initial_window,
 		    struct alist *request_types);
-struct command *
-make_server_connection_service(struct alist *session_requests);
 
 struct channel_open *
 make_open_session(struct alist *session_requests);
 
 struct channel_request *
-make_shell_handler(struct io_backend *backend,
-		   struct reap *reaper);
+make_shell_handler(struct io_backend *backend);
 
 struct channel_request *
-make_exec_handler(struct io_backend *backend,
-		  struct reap *reaper);
+make_exec_handler(struct io_backend *backend);
+
+struct channel_request *
+make_subsystem_handler(struct io_backend *backend,
+		       const char **subsystems);
 
 struct lsh_string *
 format_exit_signal(struct ssh_channel *channel,
