@@ -2,7 +2,7 @@
  *
  * Reads a sexp in given form from, and writes it in given form.
  *
- * $Id: sexp-conv.c,v 1.7 2001/04/26 05:22:34 nisse Exp $ */
+ * $Id: sexp-conv.c,v 1.8 2001/07/02 21:09:48 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -269,8 +269,8 @@ static struct sexp_conv_options *make_options(void)
   self->output = SEXP_ADVANCED;
   self->once = 0;
   self->mode = MODE_VANILLA;
-  self->select = &command_I.super;
-  self->transform = &command_I.super;
+  self->select = &command_I;
+  self->transform = &command_I;
   self->algorithms = make_alist(2,
 				ATOM_MD5, &md5_algorithm,
 				ATOM_SHA1, &sha1_algorithm,
@@ -310,7 +310,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	  default:
 	    fatal("Internal error!");
 	  case MODE_VANILLA:
-	    self->transform = &command_I.super;
+	    self->transform = &command_I;
 	    self->print = &make_sexp_print_command(self->output)->super;
 	    break;
 	  case MODE_SPKI_HASH:
@@ -325,7 +325,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	    {
 	      CAST_SUBTYPE(hash_algorithm, a,
 			   ALIST_GET(self->algorithms, self->hash));
-	      self->transform = &command_I.super;
+	      self->transform = &command_I;
 	      self->print = make_sexp_print_raw_hash(a);
 	      break;
 	    }
