@@ -2,7 +2,7 @@
  *
  * Reading and writing of s-expressions.
  *
- * $Id: sexp_commands.c,v 1.16 2001/05/01 18:25:34 nisse Exp $ */
+ * $Id: sexp_commands.c,v 1.20 2001/07/06 21:17:15 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -35,7 +35,7 @@
 
 /* Forward declarations */
 static struct catch_command catch_sexp_exceptions;
-#define CATCH_SEXP (&catch_sexp_exceptions.super.super.super)
+#define CATCH_SEXP (&catch_sexp_exceptions.super.super)
 
 static struct read_sexp_command read_sexp;
 #define READ_SEXP (&read_sexp.super.super)
@@ -50,83 +50,56 @@ static struct read_sexp_command read_sexp;
  *
  * Prints the sexp to tha abstract_write OUT. Returns the sexp. */
 
-/* GABA:
-   (class
-     (name sexp_print_to)
-     (super command)
-     (vars
-       (format . int)
-       (dest object abstract_write)))
-*/
-
 static void
-do_sexp_print(struct command *s,
-	      struct lsh_object *a,
+do_sexp_print(struct command_2 *s,
+	      struct lsh_object *a1,
+	      struct lsh_object *a2,
 	      struct command_continuation *c,
 	      struct exception_handler *e UNUSED)
 {
-  CAST(sexp_print_to, self, s);
-  CAST_SUBTYPE(sexp, o, a);
+  CAST(sexp_print_command, self, s);
+  CAST_SUBTYPE(abstract_write, dest, a1);
+  CAST_SUBTYPE(sexp, o, a2);
 
-  A_WRITE(self->dest, sexp_format(o, self->format, 0));
+  A_WRITE(dest, sexp_format(o, self->format, 0));
   if (self->format != SEXP_CANONICAL)
-    A_WRITE(self->dest, ssh_format("\n"));
+    A_WRITE(dest, ssh_format("\n"));
 
-  COMMAND_RETURN(c, a);
+  COMMAND_RETURN(c, o);
+}
+
+struct command_2 *
+make_sexp_print_command(int format)
+{
+  NEW(sexp_print_command, self);
+  self->super.super.call = do_command_2;
+  self->super.invoke = do_sexp_print;
+  self->format = format;
+
+  return &self->super;
 }
 
 struct command *
 make_sexp_print_to(int format, struct abstract_write *dest)
 {
-  NEW(sexp_print_to, self);
-  self->super.call = do_sexp_print;
-  self->format = format;
-  self->dest = dest;
-
-  return &self->super;
+  return
+    make_command_2_invoke(make_sexp_print_command(format),
+			  &dest->super);
 }
 
-struct lsh_object *
-do_sexp_print_simple(struct command_simple *s,
-		     struct lsh_object *a)
+DEFINE_COMMAND3(sexp_print_raw_hash)
+     (struct lsh_object *a1,
+      struct lsh_object *a2,
+      struct lsh_object *a3,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
-  CAST(sexp_print_command, self, s);
-  CAST_SUBTYPE(abstract_write, dest, a);
-
-  return &make_sexp_print_to(self->format, dest)->super;
-}
-
-struct command_simple *
-make_sexp_print_command(int format)
-{
-  NEW(sexp_print_command, self);
-  self->super.super.call = do_call_simple_command;
-  self->super.call_simple = do_sexp_print_simple;
-  self->format = format;
-
-  return &self->super;
-}
-
-/* GABA:
-   (class
-     (name sexp_print_raw_hash_to)
-     (super command)
-     (vars
-       (algorithm object hash_algorithm)
-       (dest object abstract_write)))
-*/
-
-static void
-do_print_raw_hash_to(struct command *s,
-		     struct lsh_object *a,
-		     struct command_continuation *c,
-		     struct exception_handler *e UNUSED)
-{
-  CAST(sexp_print_raw_hash_to, self, s);
-  CAST_SUBTYPE(sexp, o, a);
+  CAST_SUBTYPE(hash_algorithm, algorithm, a1);
+  CAST_SUBTYPE(abstract_write, dest, a2);
+  CAST_SUBTYPE(sexp, o, a3);
 
   struct lsh_string *canonical = sexp_format(o, SEXP_CANONICAL, 0);
-  struct hash_instance *hash = MAKE_HASH(self->algorithm);
+  struct hash_instance *hash = MAKE_HASH(algorithm);
   struct lsh_string *digest = lsh_string_alloc(hash->hash_size);
 
   HASH_UPDATE(hash, canonical->length, canonical->data);
@@ -135,49 +108,16 @@ do_print_raw_hash_to(struct command *s,
   lsh_string_free(canonical);
   KILL(hash);
 
-  A_WRITE(self->dest, ssh_format("%lxfS\n", digest));
+  A_WRITE(dest, ssh_format("%lxfS\n", digest));
 
-  COMMAND_RETURN(c, a);
+  COMMAND_RETURN(c, o);
 }
 
-struct command *
-make_sexp_print_raw_hash_to(struct hash_algorithm *algorithm,
-			    struct abstract_write *dest)
-{
-  NEW(sexp_print_raw_hash_to, self);
-  self->super.call = do_print_raw_hash_to;
-  self->algorithm = algorithm;
-  self->dest = dest;
-
-  return &self->super;
-}
-
-static struct lsh_object *
-collect_print_raw_hash_2(struct collect_info_2 *info,
-			 struct lsh_object *a,
-			 struct lsh_object *d)
-{
-  CAST_SUBTYPE(hash_algorithm, algorithm, a);
-  CAST_SUBTYPE(abstract_write, dest, d);
-
-  assert(!info->next);
-  
-  return &make_sexp_print_raw_hash_to(algorithm, dest)->super;
-}
-
-struct collect_info_2 collect_info_print_raw_2 =
-STATIC_COLLECT_2_FINAL(collect_print_raw_hash_2);
-
-struct collect_info_1 sexp_print_raw_hash =
-STATIC_COLLECT_1(&collect_info_print_raw_2);
-
 struct command *
 make_sexp_print_raw_hash(struct hash_algorithm *algorithm)
 {
-  CAST_SUBTYPE(command, print,
-	       make_collect_state_1(&sexp_print_raw_hash, &algorithm->super));
-  
-  return print;
+  return make_command_3_invoke(&sexp_print_raw_hash,
+			       &algorithm->super);
 }
 
 /* GABA:
@@ -320,8 +260,12 @@ static struct read_sexp_command read_sexp
 		       (proc (read_sexp file)))))))
 */
 
-DEFINE_COMMAND_SIMPLE(for_sexp_command, a)
+DEFINE_COMMAND(for_sexp_command)
+     (struct command *s UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
   CAST_SUBTYPE(command, handler, a);
-  return for_sexp(handler);
+  COMMAND_RETURN(c, for_sexp(handler));
 }
