@@ -1,6 +1,6 @@
 /* spki_commands.h
  *
- * $Id: spki_commands.h,v 1.9 2000/11/26 23:26:02 nisse Exp $ */
+ * $Id: spki_commands.h,v 1.12 2001/07/06 20:26:07 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -28,38 +28,33 @@
 #include "interact.h"
 #include "spki.h"
 
-extern struct command_simple spki_signer2verifier;
-#define SIGNER2VERIFIER (&spki_signer2verifier.super.super)
+extern struct command spki_signer2verifier;
+#define SIGNER2VERIFIER (&spki_signer2verifier.super)
 
-extern struct command_simple spki_verifier2public;
-#define VERIFIER2PUBLIC (&spki_verifier2public.super.super)
+extern struct command spki_verifier2public;
+#define VERIFIER2PUBLIC (&spki_verifier2public.super)
 
-extern struct command_simple spki_sexp2signer_command;
-#define SEXP2SIGNER (&spki_sexp2signer_command.super.super)
+extern struct command spki_sexp2signer_command;
+#define SEXP2SIGNER (&spki_sexp2signer_command.super)
 
-extern struct command_simple spki_sexp2keypair_command;
-#define SEXP2KEYPAIR (&spki_sexp2keypair_command.super.super)
+extern struct command spki_sexp2keypair_command;
+#define SEXP2KEYPAIR (&spki_sexp2keypair_command.super)
 
-extern struct command_simple spki_make_context_command;
-#define SPKI_MAKE_CONTEXT (&spki_make_context_command.super.super)
+extern struct command spki_make_context_command;
+#define SPKI_MAKE_CONTEXT (&spki_make_context_command.super)
 
-extern struct command_simple spki_read_acls_command;
-#define SPKI_READ_ACLS (&spki_read_acls_command.super.super.super)
-
-extern struct command_simple spki_read_hostkeys_command;
-#define SPKI_READ_HOSTKEYS (&spki_read_hostkeys_command.super.super)
+extern struct command spki_read_hostkeys_command;
+#define SPKI_READ_HOSTKEYS (&spki_read_hostkeys_command.super)
 
 struct command *
 make_spki_hash(int name, struct hash_algorithm *algorithm);
 
-struct command *
-make_spki_parse_key(struct alist *algorithms);
-
 struct command *
 make_spki_read_acls(struct alist *algorithms);
 
 struct command *
 make_spki_read_userkeys(struct alist *algorithms,
+			struct alist *signature_algorithms,
 			struct interact *tty);
 			
 
