@@ -2,7 +2,7 @@
  *
  * zlib compression algorithm
  * 
- * $Id: zlib.c,v 1.30.2.1 2001/09/26 16:13:32 nisse Exp $ */
+ * $Id: zlib.c,v 1.31 2001/08/26 14:57:50 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -78,7 +78,7 @@ static const struct zlib_type
 zlib_inflate = {  inflateEnd, "inflate" };
 
 static const struct zlib_type
-zlib_deflate = {  deflateEnd, "deflate" };
+zlib_deflate = {  inflateEnd, "deflate" };
 
 /* zlib memory functions */
 static void *
@@ -205,10 +205,13 @@ do_zlib(struct compress_instance *c,
       switch (rc)
 	{
 	case Z_BUF_ERROR:
-	  werror("do_zlib (%z): Z_BUF_ERROR (probably harmless),\n"
-		 "  avail_in = %i, avail_out = %i\n",
-		 ZLIB_TYPE(&self->z)->operation,
-		 self->z.avail_in, self->z.avail_out);
+	  /* If avail_in is zero, this just means that all data have
+	   * been flushed. */
+	  if (self->z.avail_in)
+	    werror("do_zlib (%z): Z_BUF_ERROR (probably harmless),\n"
+		   "  avail_in = %i, avail_out = %i\n",
+		   ZLIB_TYPE(&self->z)->operation,
+		   self->z.avail_in, self->z.avail_out);
 	  /* Fall through */
 	case Z_OK:
 	  break;
